@@ -157,6 +157,9 @@ namespace DDM {
         // connect login signal
         connect(m_socketServer, &SocketServer::login, this, &Display::login);
 
+        // connect logout signal
+        connect(m_socketServer, &SocketServer::logout, this, &Display::logout);
+
         // connect unlock signal
         connect(m_socketServer, &SocketServer::unlock,this, &Display::unlock);
 
@@ -178,7 +181,7 @@ namespace DDM {
         connect(m_greeter, &Greeter::displayServerFailed, this, &Display::displayServerFailed);
         connect(m_greeter, &Greeter::greeterStarted, this, [this] {
             if (m_currentAuth) {
-                switchToUser(m_currentAuth->user());
+                switchToUser(m_currentAuth->user(), m_currentAuth->xdgSessionId());
             }
         });
     }
@@ -217,7 +220,7 @@ namespace DDM {
         return m_seat;
     }
 
-    void Display::switchToUser(const QString &user) {
+    void Display::switchToUser(const QString &user, int xdgSessionId) {
         if (user == "dde") {
             m_greeter->setUserActivated(false);
         }
@@ -236,23 +239,11 @@ namespace DDM {
         }
 
         auto* server = reinterpret_cast<SingleWaylandDisplayServer*>(m_displayServer);
-        server->activateUser(user);
+        server->activateUser(user, xdgSessionId);
 
         if (Logind::isAvailable()) {
             OrgFreedesktopLogin1ManagerInterface manager(Logind::serviceName(), Logind::managerPath(), QDBusConnection::systemBus());
-            auto reply = manager.ListSessions();
-            reply.waitForFinished();
-
-            const auto info = reply.value();
-            for(const SessionInfo &s : reply.value()) {
-                if (s.userName == user) {
-                    OrgFreedesktopLogin1SessionInterface session(Logind::serviceName(), s.sessionPath.path(), QDBusConnection::systemBus());
-                    if (session.state() == "online" && session.vTNr() == static_cast<uint>(auth->tty())) {
-                        session.Activate();
-                        break;
-                    }
-                }
-            }
+            manager.ActivateSession(QString::number(auth->xdgSessionId()));
         }
     }
 
@@ -396,6 +387,11 @@ namespace DDM {
         startAuth(user, password, session);
     }
 
+    void Display::logout([[maybe_unused]] QLocalSocket *socket, int id) {
+        OrgFreedesktopLogin1ManagerInterface manager(Logind::serviceName(), Logind::managerPath(), QDBusConnection::systemBus());
+        manager.TerminateSession(QString::number(id));
+    }
+
     void Display::unlock(QLocalSocket *socket,
                         const QString &user, const QString &password) {
         m_socket = socket;
@@ -590,11 +586,13 @@ namespace DDM {
         }
 
         auth->setUser(user);
+        auth->setSessionType(session.type());
+        auth->setSessionFileName(session.fileName());
         if (m_reuseSessionId.isNull()) {
             auth->setSession(session.exec());
         }
         auth->insertEnvironment(env);
-        auth->setSingleMode(m_displayServerType == DisplayServerType::SingleCompositerServerType);
+        auth->setSingleMode(session.isSingleMode());
         auth->start();
     }
 
@@ -634,12 +632,12 @@ namespace DDM {
             else
                 stateConfig.Last.User.setDefault();
             if (mainConfig.Users.RememberLastSession.get())
-                stateConfig.Last.Session.set(auth->session());
+                stateConfig.Last.Session.set(auth->sessionFileName());
             else
                 stateConfig.Last.Session.setDefault();
             stateConfig.save();
 
-            if (m_displayServerType == DisplayServerType::SingleCompositerServerType) {
+            if (auth->isSingleMode()) {
                 auto* server = reinterpret_cast<SingleWaylandDisplayServer*>(m_displayServer);
                 server->onLoginSucceeded(user);
             } else {
@@ -647,9 +645,47 @@ namespace DDM {
                     emit loginSucceeded(m_socket, user);
                     daemonApp->displayManager()->setLastSession(auth->sessionId());
                 }
-            }
+                if (auth->identifyOnly())
+                    return;
 
-            switchToUser(auth->user());
+                // Stop the original suit of displayServer since it has finished its job.
+                disconnect(m_displayServer, &DisplayServer::stopped, this, &Display::stop);
+                m_displayServer->stop();
+                delete m_displayServer;
+                m_greeter->stop();
+                delete m_greeter;
+                m_currentAuth->stop();
+                m_socketServer->stop();
+                delete m_socketServer;
+
+                // Start the target displayServer
+                m_terminalId = m_sessionTerminalId = fetchAvailableVt();
+                m_started = false;
+
+                if (auth->sessionType() == Session::X11Session) {
+                    m_displayServer = new XorgDisplayServer(this);
+                    m_displayServerType = X11DisplayServerType;
+                } else {
+                    m_displayServer = new WaylandDisplayServer(this);
+                    m_displayServerType = WaylandDisplayServerType;
+                }
+                connect(m_displayServer, &DisplayServer::started, this, &Display::displayServerStarted);
+                connect(m_displayServer, &DisplayServer::stopped, this, &Display::stop);
+
+                m_socketServer = new SocketServer(this);
+
+                // The greeter here is used to start user session.
+                m_greeter = new Greeter(this);
+                m_greeter->setDisplayServerCommand(auth->session());
+                m_greeter->setUser(user);
+                m_greeter->setSkipAuth();
+
+                connect(m_greeter, &Greeter::failed, this, &Display::stop);
+                connect(m_greeter, &Greeter::displayServerFailed, this, &Display::displayServerFailed);
+                connect(m_greeter, &Greeter::succeed, this, &Display::stop);
+
+                m_displayServer->start();
+            }
         }
         m_socket = nullptr;
     }
@@ -703,7 +739,7 @@ namespace DDM {
             if (m_displayServerType == DisplayServerType::SingleCompositerServerType) {
                 auto* server = reinterpret_cast<SingleWaylandDisplayServer*>(m_displayServer);
                 // TODO: switch to greeter
-                server->activateUser("dde");
+                server->activateUser("dde", 0);
 
                 // TODO: only current user logout will reset greeter state
                 m_greeter->setUserActivated(false);
@@ -711,7 +747,8 @@ namespace DDM {
             }
         }
 
-        if (status != Auth::HELPER_AUTH_ERROR && m_displayServerType != DisplayServerType::SingleCompositerServerType)
+        // Don't restart display when ddm-helper crashed (exit code 9)
+        if (status != Auth::HELPER_AUTH_ERROR && status != Auth::HelperExitStatus(9) && m_displayServerType != DisplayServerType::SingleCompositerServerType)
             stop();
     }
 
@@ -727,11 +764,12 @@ namespace DDM {
         }
     }
 
-    void Display::slotSessionStarted(bool success) {
+    void Display::slotSessionStarted(bool success, int xdgSessionId) {
         qDebug() << "Session started" << success;
         Auth* auth = qobject_cast<Auth*>(sender());
-        if (m_displayServerType == SingleCompositerServerType) {
-            switchToUser(auth->user());
+        auth->setXdgSessionId(xdgSessionId);
+        if (auth->isSingleMode()) {
+            switchToUser(auth->user(), xdgSessionId);
         }
 
         if (success && m_displayServerType != SingleCompositerServerType) {
