@@ -42,6 +42,8 @@ namespace DDM {
         void setSocket(const QString &socket);
         void setTheme(const QString &theme);
         void setSingleMode(bool on = true);
+        void setUser(const QString &user);
+        void setSkipAuth(bool on = true);
         void setUserActivated(bool active);
 
         QString displayServerCommand() const;
@@ -55,7 +57,7 @@ namespace DDM {
 
     private slots:
         void onRequestChanged();
-        void onSessionStarted(bool success);
+        void onSessionStarted(bool success, int xdgSessionId);
         void onDisplayServerReady(const QString &displayName);
         void onHelperFinished(Auth::HelperExitStatus status);
         void onReadyReadStandardOutput();
@@ -67,6 +69,7 @@ namespace DDM {
         void ttyFailed();
         void failed();
         void displayServerFailed();
+        void succeed();
         void greeterStarted();
 
     private:
@@ -75,6 +78,8 @@ namespace DDM {
         bool m_userActivated { false };
         int m_currentRetry { 0 };
         int m_maxRetry{ 3 };
+        QString m_user = QStringLiteral("dde");
+        bool m_skipAuth { false };
 
         Display * const m_display { nullptr };
         QString m_socket;
